/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

import java.awt.event.*;
import java.awt.*;
import javax.swing.*;
import javax.swing.border.*;
import java.util.Vector;
import java.io.File;
import javax.swing.table.*;

public class FrameFourier extends JInternalFrame 
{
	protected static final int	xOffset = 30, yOffset = 30;
	protected static int		contadorDeInstancias = 0;
	protected BaseFourier		baseDados;

	// Tamanho mximo de sequencia aceito para transformada
	// de Fourier por limitaes das mquinas
	protected int	TAMANHO = 15;
	
	protected Container			c;
	protected Font				fonteArial10;
	protected FourierTask		tarefa;
    protected Timer				timer;
	protected ImageIcon			imgTopo;
	protected JProgressBar		status;
	protected JLabel			labelTopo;
	protected JFileChooser		seletorArquivos;
	protected JButton			btAbrir;
	protected Complexos			sequencia[];
	protected JButton			btTransformar;
	protected JTable			tabelaBaseDados;
	protected JTable			tabelaResultados;
	protected JScrollPane		scrollResultados;
	protected JScrollPane		scrollDados;
	protected JButton			btFechar;
	protected JPanel			painelBotoes;
	protected JPanel			painelStatus;
	protected JPanel		    painelDados;
	protected JPanel			painelResultados;
	protected JPanel			painelAvisos;
	
	public FrameFourier() 
	{
        super("Algoritmo da TRANSFORMADA DE FOURIER #" + (++contadorDeInstancias), 
              false,	// manipulvel
              true,		// fechvel
              false,	// maximizvel
              true);	// iconificvel
		
		fonteArial10	= new Font("Arial", Font.PLAIN, 10);
		
		setSize(640,480);
        setLocation(xOffset*contadorDeInstancias, yOffset*contadorDeInstancias);
		setOpaque(false);
		setBackground(Color.white);

		imgTopo			 = new ImageIcon(this.getClass().getResource("pics/fourierLogo.png"));
		labelTopo		 = new JLabel(imgTopo);
		seletorArquivos  = new JFileChooser("baseNumerica.fou");
		btAbrir			 = new JButton("Abrir...");
		btTransformar	 = new JButton("Transformar");
		tarefa			 = new FourierTask();
		status			 = new JProgressBar(0, tarefa.getTamanhoTarefa());
		btFechar		 = new JButton("Fechar");
		painelBotoes	 = new JPanel();
		painelStatus	 = new JPanel();
		painelDados		 = new JPanel();
		painelResultados = new JPanel();
		painelAvisos	 = new JPanel();
		
		Border espaco = BorderFactory.createEmptyBorder(5,5,5,5);
		painelBotoes.setBorder(BorderFactory.createCompoundBorder
							  (criarBorda("OPES RPIDAS"), espaco));
		painelStatus.setBorder(BorderFactory.createCompoundBorder
							  (criarBorda("PROGRESSO DA EXECUO"), espaco));
		painelDados.setBorder(BorderFactory.createCompoundBorder
							 (criarBorda("SEQNCIA INICIAL"), espaco));
		painelResultados.setBorder(BorderFactory.createCompoundBorder
								  (criarBorda("SEQNCIA TRANSFORMADA"), espaco));

		painelAvisos.setBorder(BorderFactory.createCompoundBorder
								  (criarBorda("OBSERVAES"), espaco));

		painelBotoes.setLayout(new GridLayout(1,3));
		painelStatus.setLayout(new GridLayout(1,1));
		painelDados.setLayout(new GridLayout(1,1));
		painelResultados.setLayout(new GridLayout(1,1));
		painelAvisos.setLayout(new GridLayout(1,1));

		btFechar.setFont(fonteArial10);
		btTransformar.setFont(fonteArial10);
		btAbrir.setFont(fonteArial10);
		btFechar.setMnemonic('F');
		btTransformar.setMnemonic('T');
		btAbrir.setMnemonic('A');
		btAbrir.setToolTipText("Abrir um novo arquivo base para transformao (*.fou)");
		btTransformar.setToolTipText("Inicia a transformada de Fourier");
		btFechar.setToolTipText("Fecha este quadro do Algoritmo de Fourier");
		
		c = getContentPane();
		c.setLayout(null);

		// Carrega base inicial de dados para que no fique sem nada
		carregarBaseDados("baseNumerica.fou");
		apresentarTabela();
		atualizarDados(tabelaBaseDados);

		labelTopo.setBounds		  (100, 0,   400, 40 );
		painelBotoes.setBounds	  (10,  45,  300, 60 );
		painelStatus.setBounds	  (10,  110, 300, 60 );
		painelDados.setBounds     (10,  180, 300, 260);
		painelResultados.setBounds(320, 180, 300, 260);
		painelAvisos.setBounds	  (320, 45,  300, 125);

		// Setando cores dos componentes
		painelBotoes.setBackground(Color.white);
		painelStatus.setBackground(Color.white);
		painelDados.setBackground(Color.white);
		painelAvisos.setBackground(Color.white);
		painelResultados.setBackground(Color.white);
		status.setForeground(new Color(18, 126, 201));

		painelBotoes.add(btAbrir);
		painelBotoes.add(btTransformar);
		painelBotoes.add(btFechar);
		painelStatus.add(status);
		painelDados.add(scrollDados);
		painelResultados.add(scrollResultados);
		painelAvisos.add(criarObservacoes());

		// Adicionando ao frame
		c.add(labelTopo);
		c.add(painelBotoes);
		c.add(painelStatus);
		c.add(painelDados);
		c.add(painelResultados);
		c.add(painelAvisos);

		status.setValue(0);
		status.setStringPainted(true);
		timer = criarTimer();
		adicionarListeners();
	}

	private JScrollPane criarObservacoes()
	{
		String observacao = "";
		
		observacao = "- A implementao do algoritmo aqui apresentada \n"
				   + "  tem fins didticos, apenas.\n"	
				   + "\n- O nmero de entradas foi limitado em 15 para \n"
				   + "  simplificaes e por questes de limitaes  \n"
				   + "  das mquinas alvo. \n"
				   + "\n- A extenso de arquivo BASE foi padronizada \n"
				   + "  em \'*.fou\' para simplificao de projeto.\n"
				   + "\n- Qualquer arquivo \'*.fou\' deve ser escrito no formato \n"
				   + "    a) Dois nmeros por linhas separados por TAB \n"
				   + "        ou ESPAO \n"
				   + "    b) Cada linha representa um NMERO COMPLEXO \n"
				   + "        o qual ser uma das entradas para o algoritmo. \n"
				   + "    c) O primeiro nmero representa a parte REAL \n"
				   + "    d) O segundo nmero representa a parte \n"
				   + "        IMAGINRIA \n";

		JTextArea aviso = new JTextArea(observacao);
		JScrollPane scrollAviso = new JScrollPane(aviso);
		
		scrollAviso.setBorder(BorderFactory.createEmptyBorder());
		scrollAviso.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
		scrollAviso.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);

		aviso.setFont(fonteArial10);
		aviso.setEditable(false);

		return scrollAviso;
	}
	
	private Border criarBorda(String titulo)
	{
		Border bd = BorderFactory.createTitledBorder(
					BorderFactory.createEtchedBorder(),
					titulo, TitledBorder.LEFT, 
					TitledBorder.TOP, fonteArial10);

		return bd;
	}

	private void apresentarTabela()
	{
		tabelaBaseDados = new JTable(modelarTabela());
		tabelaResultados = new JTable(modelarTabela());
		scrollDados = new JScrollPane(tabelaBaseDados);
		scrollResultados = new JScrollPane(tabelaResultados);
		
		scrollDados.setBackground(Color.white);
		tabelaBaseDados.setBackground(Color.white);
		tabelaBaseDados.setFont(fonteArial10);
		tabelaBaseDados.getTableHeader().setFont(fonteArial10);
		scrollResultados.setBackground(Color.white);
		tabelaResultados.setBackground(Color.white);
		tabelaResultados.setFont(fonteArial10);
		tabelaResultados.getTableHeader().setFont(fonteArial10);
		DefaultTableCellRenderer render = new DefaultTableCellRenderer() {};

		TableColumn real1		= tabelaBaseDados.getColumn("REAL");
		TableColumn imaginaria1 = tabelaBaseDados.getColumn("IMAGINARIA");
		TableColumn real2		= tabelaResultados.getColumn("REAL");
		TableColumn imaginaria2 = tabelaResultados.getColumn("IMAGINARIA");

		render.setHorizontalAlignment(JLabel.LEFT);
		render.setHorizontalAlignment(JLabel.LEFT);
		real1.setCellRenderer(render);
		imaginaria1.setCellRenderer(render);
		real2.setCellRenderer(render);
		imaginaria2.setCellRenderer(render);
	}
	
	private void atualizarDados(JTable tabela)
	{
		String strReal, strImg;

		if (TAMANHO > sequencia.length)
		{
			int diferenca = TAMANHO - sequencia.length;

			for (int i = 0; i < sequencia.length; i++)
			{
				strReal = sequencia[i].getReal() + "";
				strImg = sequencia[i].getImaginaria() + "";
				tabela.setValueAt(strReal, i, 0);
				tabela.setValueAt(strImg, i, 1);
			}

			for (int i = TAMANHO - diferenca; i < TAMANHO; i++)
			{
				tabela.setValueAt("No disponvel", i, 0);
				tabela.setValueAt("No disponvel", i, 1);
			}
			repaint();
		}
		else
		{
			for (int i = 0; i < TAMANHO; i++)
			{
				strReal = sequencia[i].getReal() + "";
				strImg = sequencia[i].getImaginaria() + "";
				tabela.setValueAt(strReal, i, 0);
				tabela.setValueAt(strImg, i, 1);
			}
			repaint();
		}
	}
	
	private boolean carregarBaseDados(String arquivoSelecionado)
	{
		Vector vetorObjetos = null;

		try
		{
			baseDados = new BaseFourier(arquivoSelecionado);
			vetorObjetos = baseDados.getSequenciaNumeros();

			if (vetorObjetos.size() < TAMANHO)
				sequencia = new Complexos[vetorObjetos.size()];
			else
				sequencia = new Complexos[TAMANHO];

			for (int i = 0; i < sequencia.length; i++)
			{
				sequencia[i] = new Complexos();
				sequencia[i].recebe((Complexos)vetorObjetos.elementAt(i));
			}
		}
		catch (Exception e) {sequencia = null;}

		if (sequencia != null)
			return true;
		
		return false;
	}

	private void transformar()
	{
		try
		{					
			timer.start();
			status.setValue(0);
			tarefa.setSequencia(sequencia);
			tarefa.go();
			tarefa.setMensagem("");
		}
		catch (Exception ex) {}
	}

	private void abrir()
	{
		int retorno = seletorArquivos.showDialog(FrameFourier.this, 
					  "Selecionar base de dados");

		if (retorno == JFileChooser.APPROVE_OPTION) 
		{
			File file = seletorArquivos.getSelectedFile();

			if (carregarBaseDados(file.getName()))
				atualizarDados(tabelaBaseDados);	
			else
			{
				JOptionPane.showMessageDialog(FrameFourier.this, 
							"ERRO: o arquivo \"" + file.getName() + "\" no  \n" 
							+ "uma base de dados vlida",
							"ERRO", JOptionPane.INFORMATION_MESSAGE);
			}
		} 
	}

	private Timer criarTimer()
	{
		Timer t = new Timer(100, new ActionListener() 
		{
            public void actionPerformed(ActionEvent evt)
			{
				status.setValue(tarefa.getStatus());
				
				if (tarefa.done()) 
				{
					Toolkit.getDefaultToolkit().beep();
					timer.stop();
					sequencia = tarefa.getSequencia();
					atualizarDados(tabelaResultados);
					repaint();
					status.setValue(status.getMinimum());
					btAbrir.setEnabled(true);
				}
			}
		});
		
		return t;
	}

	private void adicionarListeners()
	{
		btTransformar.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e) 
			{ 
				btTransformar.setEnabled(false);
				btAbrir.setEnabled(false);
				transformar(); 
			}
		});

		btAbrir.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e) 
			{ 
				abrir(); 
				btTransformar.setEnabled(true);
			}
		});

		btFechar.addActionListener(new ActionListener()
		{
			public void actionPerformed(ActionEvent e) { dispose(); }
		});
	}

	public TableModel modelarTabela()
	{
		final	String	names[]		= {"REAL", "IMAGINARIA"};
		final	Object  data[][]    = new Object[TAMANHO][TAMANHO];

		String str = "No disponvel";
		
		for (int i = 0; i < TAMANHO; i++) 
		{
			data[i][0] = str;
			data[i][1] = str;
		}
		
		TableModel modelo = new AbstractTableModel() 
		{
            public int getColumnCount() { return names.length; }
            public int getRowCount() { return data.length;}
            public Object getValueAt(int row, int col) {return data[row][col];}
            public String getColumnName(int column) {return names[column];}
            public Class getColumnClass(int c) {return getValueAt(0, c).getClass();}
            public boolean isCellEditable(int row, int col) {return false;}

			public void setValueAt(Object aValue, int row, int column) 
			{
				double tmp;
				Complexos c;
				String str;

				str = (String)aValue;

				if (row < sequencia.length)
				{
					try
					{
						if (column == 0)
						{
							tmp = Double.parseDouble(str);
							c = new Complexos(tmp, sequencia[row].getImaginaria());
						}
						else
						{
							tmp = Double.parseDouble(str);
							c = new Complexos(sequencia[row].getReal(), tmp);
						}

						sequencia[row] = c;
						data[row][column] = aValue;
					}
					catch(Exception e) {}
				}
				else
					data[row][column] = aValue;
			}
		};

		return modelo;
	}
}