/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

import java.lang.Math;

public class Operacoes extends Thread
{
	final int SHOR		= 1;
	final int FOURIER	= 2; 
	int ALGORITMO		= SHOR;
	
	// Constante matemtica PI
    final double PI = 3.1415926535897932384626433832795;

    // Esta funcao pega um inteiro de entrada e retorna 1 se este  
    // um primo e 0 em caso contrrio
    public boolean testarPrimalidade(int n) 
    {
        int i;
        for (i = 2 ; i <= Math.floor(Math.sqrt(n)); i++) 
        {
            if (n % i == 0)
                return false;
        }
        return true;
    }

    // Esta funo pega uma entrada inteira e retorna 1 se este  uma potncia de 
    // primos e 0 em caso contrrio
    public boolean testarPotenciaPrimos(int n) 
    {
        int i,j;
        j = 0; 
        i = 2;
        while ((i <= Math.floor(Math.pow(n, 0.5))) && (j == 0)) 
        {
            if((n % i) == 0) 
                j = i;
            i++;
        }
    
        for (i = 2; i <= (Math.floor(Math.log(n) / Math.log(j)) + 1) ; i++) 
        { 
            if(Math.pow(j , i) == n) 
                return true;
        }
        return false;
    }

    // Esta funo verifica se um dado nmero  potncia de 2 ou no
    public boolean testarPotenciaDois(int n)
    {
        double numero = n;

        while (numero > 1)
            numero /= 2;

        return (numero == 1);
    }
    
    // Computa o mximo divisor comum entre dois inteiros
    // Desde que o mdulo 0 (mod 0) de um nmero no  definido
    // retorna-se -1 como um cdigo de erro em qualquer tentativa desde
    // tipo
    public int MDC(int a, int b) 
    {
        int d;
        if (b != 0) 
        {
            while (a % b != 0) 
            {
                d = a % b;
                a = b;  
                b = d;
            }
        } 
        else 
            return -1;
        return (b);
    }

    // Esta funo toma um argumento inteiro e retorna o nmero de bits necessrios
    // para sua representacao quantica
    public int tamanhoRegistrador(int a) 
    {
        int tamanho = 0;
        while (a != 0) 
        {
            a = a >> 1;
            tamanho++;
        }
        return (tamanho);
    }

    // q  a potncia de 2 tal que n^2 <= q < 2n^2
    public int GetQ(int n) 
    {
        int potencia = 8; // 256  o menor q que existe
        
        while (Math.pow(2,potencia) < Math.pow(n,2)) 
            potencia = potencia + 1;

        return ((int)Math.pow(2,potencia));
    }

    // Esta funo toma trs inteiros, [x,a,n] e retorna x^a mod n
    // Este algoritmo  conhecido com: "RUSSIAN PEASANT METHOD"
    public int modexp(int x, int a, int n) 
    {
        int valor = 1;
        int tmp;
        tmp = x % n;
        while (a > 0) 
        {
            if ((a&1) != 0)
                valor = (valor * tmp) % n;
            tmp = tmp * tmp % n;
            a = a >> 1;
        }
        return valor;
    }

    // Esta funo acha um denominador q do melhor numero racional
    // a partir da aproximao p / q para para c tal que q < qmax
    public int denominador(double c, int qmax) 
    {
        double y = c;
        double z;
        int q0 = 0;
        int q1 = 1;
        int q2 = 0;

        while (true) 
        {
            z = y - Math.floor(y);
            if (z < 0.5 / Math.pow(qmax,2)) 
                return(q1); 
            if (z != 0) 
            {
                // Esta diviso no pode ter Z como ZERO
                y = 1 / z;
            } 
            else 
              return (q1);
            
            q2 = (int)Math.floor(y) * q1 + q0;
            
            if (q2 >= qmax) 
              return (q1); 

            q0 = q1; 
            q1 = q2;
        }
    }

    // Retorna o maior entre dois numeros
    int maior(int a, int b) 
    {
        if (a > b) 
            return(a);
        
        return(b);
    }

    // Computa a transformao quntica de FOURIER. Mapeia q-1 entradas
    void transformacaoFourier(Registradores reg, int q) 
    {       
		// A TQF mapeia funes no domnio do tempo para funes no 
        // domnio da frequncia. Sabemos que a frequncia  1/perodo
        // Desta forma a TQF ir pegar um registrador que ir simular
        // a memria quntica e fazer os clculos sobre este.
        // Matematicamente a TQF para um estado q :
        // q^(-0.5) * Soma[c = 0 -> c = q - 1, c * e^(2*Pi*i*a*c / q)]
        // Relembrando que e^ix = cos x + sen x * i

        Complexos temporario[];
        temporario = new Complexos[q];

        for (int j = 0; j < temporario.length; j++)
            temporario[j] = new Complexos();

        Complexos tmpComp = new Complexos();
        tmpComp.set(0,0);

        // Aqui ns passamos a fazer operaes que no so fisicamente
        // realizveis pois estamos manipulando / acessando estados
        // qunticos sem colaps-los. Existe, no entanto, uma boa notcia:
        // numa possvel, embora remota, implementao REAL de um 
        // computador quntico poderamos construir circuitos qunticos
        // para tais funes
        int contador = 0;
        double tmpReal = 0;
        double tmpImag = 0;
        double tmpProb = 0;

		for (int a = 0 ; a < q ; a++) 
        {
			// Estes procedimentos ajudam a prevenir quanto a erros de 
            // arredondamento anteriores
            if ((Math.pow(reg.getProbabilidade(a).getReal(), 2) + 
                 Math.pow(reg.getProbabilidade(a).getImaginaria(),2)) > Math.pow(10,-15)) 
            {
                for (int c = 0 ; c < q ; c++) 
                {
                    tmpComp.set(Math.pow(q,-0.5) * Math.cos(2*PI*a*c/q),
                                Math.pow(q,-0.5) * Math.sin(2*PI*a*c/q));
                    
                    Complexos mult = new Complexos();
                    mult = (reg.getProbabilidade(a)).mult(tmpComp);
                    temporario[c] = temporario[c].soma(mult);
                }
            }
            contador++;
			
			if (contador == 100) 
            {
				double status = 100*((double)a / (double)(q - 1));
                contador = 0;

				switch (ALGORITMO)
				{
					case SHOR:
						ShorTask.setStatus(status);
						break;
					case FOURIER:
						try 
						{
							FourierTask.setStatus(status);
							// Isto parece estranho mas  apenas para evitar
							// clculos errados e tambm para se ter um acompanhamento
							// visual do que est acontecendo
							sleep(100);
						}
						catch(Exception exc){}
						break;
					default:
						break;
				}
            }
        }
        
 		reg.setEstado(temporario);
		reg.normalizar();
    }
}
