/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

// Classe QuBit, define as propriedades de um quBit
// Um quBit  representado fisicamente por [alfa * |0> + beta * |1>]

import java.lang.String;
import java.lang.Math;
import java.util.Random;

public class QuBit
{
    /* METODOS DISPONIVEIS
        + QuBit()
        + String espiar()
        + int medir()
        + void setEstado(Complexos probZero, Complexos probUm)
        + void setMedia()
        + double modulo(int estado)
    */ 
    
    // Probabilidade de achar um QuBit no estado ZERO ou em toda 
    // a parte imaginaria
	private Complexos alfa;
    // Probabilidade de achar um QuBit no estado Um ou em toda
    // a parte real
    private Complexos beta;
    // OBS.: A raiz quadrada de probZero + raiz quadrada do modulo de 
    // probUm precisa ser sempre 1. 
    // Em resumo: (|alfa|)^2 + (|beta|)^2
      
    // Construtor padrao de um QuBit
	public QuBit()
    {
        alfa.set(1,0);
        beta.set(0,0);
    }
    
    // Mede os estados sem colaps-los. Obviamente em um computador
    // quntico real este mtodo no seria realizvel fisicamente dado 
    // que  impossvel "ler" os estados qunticos sem colaps-los.
    public String espiar()
    {
        String str = new String("");
        str = "Amplitude de alfa: " + alfa.getReal() + " + " + alfa.getImaginaria() + "i";
        str += " | Amplitude de beta " + beta.getReal() + " + "  + beta.getImaginaria() + "i";
        return str;
    }
    
    // Retorna o estado zero com probabilidade |alfa|^2  ou com probabilidade |beta|^2
    // Corresponde a medir a componente Z do SPIN.
    public int medicao()
    {
        Random rand = new Random();
        double nRand = rand.nextDouble();
    
		// Supondo que a soma dos quadrados das amplitudes seja = 1
        if (modulo(0) > nRand)
        {
            // Seria preciso colapsar as fases?
            alfa.set(1,0);
            beta.set(0,0);
            return 0;
        } 
        else 
        {
            alfa.set(0,0);
            beta.set(1,0);            
            return 1;
        } 
    }
    
	// Seta um estado a partir de dois numeros complexos representando 
    // as probabilidades
    public void setEstado(Complexos probZero, Complexos probUm)
    {
        alfa = probZero;
        beta = probUm;
        // Determina se |alfa|^2 + |beta|^2 == 1.
        // Caso isso no seja validado tem-se um estado invalido
        double prob;
        prob = modulo(0) + modulo(1);
        
        if (Math.abs(prob - 1) > Math.pow(10, -9))
            System.err.println("AVISO! Probabilidade TOTAL diferente de 1 (um)");
    }
    
    // Seta o alfa  e beta para 2^(0.5). Como no h mais 
    // parte imaginaria no h componente de fase
    public void setMedia()
    {
        alfa.set(Math.pow(2,-0.5), 0);
        beta.set(Math.pow(2,-0.5), 0);
    }

    // alfa * complexoConjugado(alfa) e retorna o valor. 
    // ou beta * complexoConjugado(beta) e retorna o valor
    // OBS.: Isto ser sempre um nmero real. 
    public double modulo(int estado)
    {
        if (estado == 0)
            return (Math.pow(alfa.getReal(), 2) + Math.pow(alfa.getImaginaria(), 2));
        else
            return (Math.pow(beta.getReal(), 2) + Math.pow(beta.getImaginaria(), 2));    
    }
}