/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.border.*;

public class SAQua extends JApplet
{
	protected static final int	UM_SEGUNDO = 1000;

	protected Color			c;
	protected JDesktopPane	        desktop;
	protected JLabel		labelCopyright;
	protected ImageIcon		imgCopyright;
	protected ImageIcon		iconArquivo;		
	protected ImageIcon		iconSobre;		
        protected ImageIcon             iconDeutsch;
	protected ImageIcon		iconSair;		
	protected ImageIcon		iconAlgoritmos;	
	protected ImageIcon		iconShor; 
        protected ImageIcon             iconFourier;
        protected ImageIcon             iconAjuda;
	protected JToolBar		toolBar;
	protected ImageIcon		iconLogo;
	protected JLabel		labelLogo;

	public void init()
	{
		Dimension tamanhoTela = Toolkit.getDefaultToolkit().getScreenSize();

		imgCopyright	= new ImageIcon(this.getClass().getResource("pics/copyright.png"));
		labelCopyright	= new JLabel(imgCopyright);

		// (x,y iniciais)
		// (tamanho em x e em y iniciais)
		setBounds(0, 0, tamanhoTela.width, tamanhoTela.height);

		desktop = new JDesktopPane();
		desktop.setBackground(new Color(18, 126, 201));

		setContentPane(desktop);
		Container c = getContentPane();
		
		c.setLayout(null);

		// Pegando a rea til
		Insets insets = c.getInsets();

		// Criando cones
		criarIcones();
		// Adicionando menu ao DESKTOP
		setJMenuBar(criarMenus());

		toolBar = criarToolBar();	
		
		// Adicionando botes ao DESKTOP
		labelCopyright.setBounds(tamanhoTela.width - 25, 10, 20, 354);
		toolBar.setBounds		(insets.left, insets.top, 40, 80);

		if (tamanhoTela.width > 1200)
		{
			iconLogo  = new ImageIcon(this.getClass().getResource("pics/logoSaqua1024x768.png"));
			labelLogo = new JLabel(iconLogo);
			labelLogo.setBounds	(415, 200, 450, 450);
		}
		else if (tamanhoTela.width > 1000)
		{
			iconLogo  = new ImageIcon(this.getClass().getResource("pics/logoSaqua1024x768.png"));
			labelLogo = new JLabel(iconLogo);
			labelLogo.setBounds	(287, 118, 450, 450);
		}
		else
		{
		    iconLogo  = new ImageIcon(this.getClass().getResource("pics/logoSaqua800x600.png"));
		    labelLogo = new JLabel(iconLogo);
		    labelLogo.setBounds	(200, 65, 400, 400);
		}
		
		c.add(labelCopyright);
		c.add(toolBar);
		c.add(labelLogo);
	}

	private JToolBar criarToolBar()
	{
	    JToolBar toolBar	= new JToolBar();
	    JButton btShor	= new JButton(iconShor);
	    JButton btFourier   = new JButton(iconFourier);
	    JButton btDeutsch   = new JButton(iconDeutsch);

	    btShor.setToolTipText("Acessa o algoritmo de fatorao Peter Shor");
	    btFourier.setToolTipText("Acessa o algoritmo da Transformada de Fourier");
	    btDeutsch.setToolTipText("Simula o problema de Deutsch" + 
				     "ou \"The Deutsch's Problem\"");

	    btShor.addActionListener(new ActionListener()
	    {
			public void actionPerformed(ActionEvent e)
			{
				criarFrameShor();
			}
	    });

	    btFourier.addActionListener(new ActionListener()
		{
		    public void actionPerformed(ActionEvent e)
		    {
			criarFrameFourier();
		    }
		});

	    btDeutsch.addActionListener(new ActionListener()
		{
		    public void actionPerformed(ActionEvent e)
		    {
			criarFrameDeutsch();
		    }
		});

	    toolBar.setLayout(new GridLayout(3,1));
	    toolBar.add(btShor);
	    toolBar.add(btFourier);
	    toolBar.add(btDeutsch);
	    toolBar.setFloatable(false);

	    toolBar.setBorder(BorderFactory.createEtchedBorder());
	    
	    return toolBar;
	}

    private void criarIcones()
    {
	iconArquivo		= new ImageIcon(this.getClass().getResource("pics/arquivo.png"));
	iconSobre		= new ImageIcon(this.getClass().getResource("pics/sobre.png"));
	iconSair		= new ImageIcon(this.getClass().getResource("pics/sair.png"));
	iconAlgoritmos	= new ImageIcon(this.getClass().getResource("pics/algoritmos.png"));
	iconShor	    = new ImageIcon(this.getClass().getResource("pics/shor.png"));
	iconFourier	    = new ImageIcon(this.getClass().getResource("pics/fourier.png"));
        iconDeutsch     = new ImageIcon(this.getClass().getResource("pics/deutsch.png"));
        iconAjuda       = new ImageIcon(this.getClass().getResource("pics/ajuda.png"));
    }

    private JMenuBar criarMenus()
    {
	c = new Color(205, 200, 200);
	
	// Declarando a barra de menus
	Font	 fonteArial10;
	JMenuBar bar;
	JMenu	menuArquivo;
	JMenu	menuAlgoritmos;
        JMenu   menuAjuda;
	JMenuItem itemSobre;
        JMenuItem itemSair;
        JMenuItem itemShor;
	JMenuItem itemFourier;
        JMenuItem itemDeutsch;

	// Criando a barra de menus e
        // Instanciando os objetos do menu
	bar				= new JMenuBar();
        menuArquivo		= new JMenu("ARQUIVO");
	menuAlgoritmos	= new JMenu("ALGORITMOS");
        menuAjuda       = new JMenu("AJUDA");
	itemSobre		= new JMenuItem("Sobre o Desenvolvimento");
        itemSair		= new JMenuItem("Sair");
        itemShor		= new JMenuItem("Fatorao de Peter Shor");
	itemFourier		= new JMenuItem("Transformada de Fourier");
        itemDeutsch     = new JMenuItem("O Problema de Deutsch");
	fonteArial10    = new Font("Arial", Font.BOLD, 10);
		
	// Escolhendo o mnemonico para a opo
        menuArquivo.setMnemonic('A');
	menuAlgoritmos.setMnemonic('L');
        menuAjuda.setMnemonic('J');
        itemSobre.setMnemonic('O');
        itemSair.setMnemonic('S');
	itemShor.setMnemonic('S');
	itemFourier.setMnemonic('F');
        itemDeutsch.setMnemonic('P');

	// Tooltips
	itemSobre.setToolTipText("Informaes sobre os desenvolvedores");
	itemSair.setToolTipText("Efetua a sada do sistema");
	itemShor.setToolTipText("Acessa o algoritmo de fatorao Peter Shor");
	itemFourier.setToolTipText("Acessa o algoritmo da Transformada de Fourier");
        itemDeutsch.setToolTipText("Simula o problema de Deutsch" + 
                                   "ou \"The Deutsch's Problem\"");
	menuAjuda.setToolTipText("Ajuda sobre o SAQua");

	// Setando os icones
	menuArquivo.setIcon(iconArquivo);
	menuAlgoritmos.setIcon(iconAlgoritmos);
        menuAjuda.setIcon(iconAjuda);
	itemSair.setIcon(iconSair);
	itemSobre.setIcon(iconSobre);
	itemShor.setIcon(iconShor);
	itemFourier.setIcon(iconFourier);
        itemDeutsch.setIcon(iconDeutsch);
		
	// Setando o fundo de cada componente
	bar.setBackground(c);
	menuArquivo.setBackground(c);
	menuAlgoritmos.setBackground(c);
        menuAjuda.setBackground(c);
	itemSair.setBackground(c);
	itemShor.setBackground(c);
	itemSobre.setBackground(c);
	itemFourier.setBackground(c);
	itemDeutsch.setBackground(c);

        itemSair.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S, ActionEvent.CTRL_MASK));
	itemSobre.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_D, ActionEvent.CTRL_MASK));
	itemShor.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_1, ActionEvent.CTRL_MASK));
	itemFourier.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_2, ActionEvent.CTRL_MASK));
        itemDeutsch.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_3, ActionEvent.CTRL_MASK));
        
	// Listeners para as aes
	itemSobre.addActionListener(new HandlerMenu(SAQua.this, this));
        itemSair.addActionListener(new HandlerMenu(SAQua.this, this));
	itemShor.addActionListener(new HandlerMenu(SAQua.this, this));
	itemFourier.addActionListener(new HandlerMenu(SAQua.this, this));
	itemDeutsch.addActionListener(new HandlerMenu(SAQua.this, this));
	menuAjuda.addActionListener(new HandlerMenu(SAQua.this, this));

	// Adicionando fisicamente aos menus
	menuArquivo.add(itemSair);
	menuAlgoritmos.add(itemShor);
        menuAlgoritmos.addSeparator();
	menuAlgoritmos.add(itemFourier);
        menuAlgoritmos.addSeparator();
        menuAlgoritmos.add(itemDeutsch);
	menuAjuda.add(itemSobre);
		
	// Adicionando fisicamente a barra de menus
	bar.add(menuArquivo);
	bar.add(menuAlgoritmos);	
        bar.add(menuAjuda);

	menuArquivo.setFont(fonteArial10);
	menuAlgoritmos.setFont(fonteArial10);
        menuAjuda.setFont(fonteArial10);
        itemSair.setFont(fonteArial10);
	itemSobre.setFont(fonteArial10);
	itemShor.setFont(fonteArial10);
	itemFourier.setFont(fonteArial10);
        itemDeutsch.setFont(fonteArial10);

	return bar;
    }

    protected void criarFrameFourier()
    {
	FrameFourier fFourier = new FrameFourier();
	fFourier.setVisible(true);
	desktop.add(fFourier);
	
	try 
	    {
		fFourier.setSelected(true);
	    } 
	catch (java.beans.PropertyVetoException e) {}
    }
    
    protected void criarFrameSobre()
    {
	FrameSobre fSobre = new FrameSobre();
	fSobre.setVisible(true);
	desktop.add(fSobre);
	
	try 
	    {
		fSobre.setSelected(true);
	    } 
	catch (java.beans.PropertyVetoException e) {}
    }
    
    protected void criarFrameShor()
    {
	FrameShor fShor = new FrameShor();
	fShor.setVisible(true);
	desktop.add(fShor);
	
	try 
	    {
		fShor.setSelected(true);
	    } 
	catch (java.beans.PropertyVetoException e) {}
    }
    
    protected void criarFrameDeutsch()
    {
        FrameDeutsch fDeutsch = new FrameDeutsch();
        fDeutsch.setVisible(true);
        desktop.add(fDeutsch);
	
	try 
	    {
		fDeutsch.setSelected(true);
	    } 
	catch (java.beans.PropertyVetoException e) {}
    }
    
    
    // Mtodo principal que chama o programa  execuo
    public static void main(String args[])
    {
	try 
	    {
            UIManager.setLookAndFeel(
				     UIManager.getCrossPlatformLookAndFeelClassName());
	    } 
	catch (Exception e) { }
	
	SplashScreen splash = new SplashScreen();
	final JFrame app = new JFrame("Bem-vindo ao SAQua: Simulador de Algoritmos Qunticos");
	final SAQua objApplet = new SAQua();
	
	app.addWindowListener( new WindowAdapter()
	    {
		public void windowClosing(WindowEvent e)
		{
		    ImageIcon iconeSaida = new ImageIcon(this.getClass().getResource("pics/logoSaquaPeq.png"));
		
		    JOptionPane.showMessageDialog(app, "Saindo do SAQua\n" +
						  "Simulador de Algoritmos Qunticos", "Saindo do Simulador",
						  JOptionPane.INFORMATION_MESSAGE, iconeSaida);
				
		    System.exit(0);
		}
	    }); 
	
	objApplet.init();
	objApplet.start();
	
	app.getContentPane().add(objApplet);
	
	Dimension tamanhoTela = Toolkit.getDefaultToolkit().getScreenSize();
	app.setSize(tamanhoTela.width, tamanhoTela.height);
		
	// Exibindo o frame na tela
	app.setVisible(true);
	splash.hide();
    }

}

// Cria um splash screen, ou seja, mostra uma tela de 
// crditos na inicializao do aplicativo SAQua
class SplashScreen extends JWindow
{
    public SplashScreen()
    {
        Dimension tamanhoTela = Toolkit.getDefaultToolkit().getScreenSize();
        JLabel labelSplash = new JLabel(new ImageIcon(this.getClass().getResource("pics/splash.png")));
        int origemX = tamanhoTela.width - 560;
        int origemY = tamanhoTela.height - 300;
        setBounds(origemX / 2, origemY/2, tamanhoTela.width, tamanhoTela.height);        
        setSize(560, 300);
        Container c = getContentPane();
        c.setLayout(new BorderLayout());
        c.add(labelSplash, BorderLayout.CENTER);
        show();
    }
};

