/************************************************************************  
*   PROGRAMA DE INICIAO CIENTFICA PIBIC/CNPq                         *
*   PROJETO: Desenvolvimento de um simulador de algoritmos qunticos    *
*            utilizando a computao convencional                       *
* --------------------------------------------------------------------- *
*      Bolsista: Anderson de Rezende Rocha - DCC/UFLA                   *
*                   # undersun@comp.ufla.br         (1)                 *
*                   # phiberoptic_br@yahoo.com.br   (2)                 *
* --------------------------------------------------------------------- *
*  Orientadores: Antnio Maria Pereira de Resende                       *
*                   # tonio@comp.ufla.br                                *
*                Antonio Tavares Costa Junior                           *
*                   # antc@stout.ufla.br                                *
*************************************************************************/

import java.lang.Math;
import java.util.Random;

	/* METODOS DISPONIVEIS
        + Shor(int num)

	   OBSERVAES
		+ Implementao do algoritmo de SHOR. 
		+ Agradecimentos a idias e noes de implementao 
		  de:
		     a) Matthew Hayward [http://sds.cs.uiuc.edu/]
			 b) Berhnard Ohpner [Link no disponvel]
       
	   INFORMAES DE DISTRIBUIO	
	   ----------------------------------------------------------------
	   | Todo o cdigo deste simulador pode ser LIVREMENTE modificado |
	   | desde que, mantenha-se referncia aos autores originais      |
	   | bem como os nomes dos colabores acima citados.               |
       ----------------------------------------------------------------
	*/ 

public class Shor extends Operacoes implements Runnable
{ 
	static int NUMERO;
	Random rand;
	private boolean erro;

	public String getRestricoes()
	{
		String restricoes = new String(
			   "Bem-vindo ao SAQua - Simulador de Algoritmos Quanticos\n"
			   + "Creditos para BSc.: Anderson de Rezende Rocha\n\n"
			   + "Algoritmo de SHOR | Restricoes:\n"
			   + "1) O numero a ser fatorado precisa ser >= 15\n"
			   + "2) O numero a ser fatorado precisa ser impar\n"
			   + "3) O numero precisa ser nao-primo\n"
			   + "4) O numero precisa nao ser uma potencia de primos\n\n"
			   + "FATORANDO " + NUMERO + "\n\n");

		return restricoes;
	}

	Shor(int num)
	{
		// Gerando uma referncia para um nmero aleatrio
		rand = new Random();
		// Numero 21  padro para fatorao
		NUMERO = num;
		erro = false;
		ALGORITMO = SHOR;
	}

	private boolean executarAlgoritmo()
	{
		int tentativas = 1;
		
		ShorTask.setMensagem(getRestricoes());

		// RESTRIO do algoritmo de SHOR #1:
		// + O nmero precisa ser IMPAR
		if (NUMERO % 2 == 0) 
		{
			ShorTask.setMensagem("AVISO #1: o numero precisa ser impar!\n");
			erro = true;
			return false;
		} 
		  
		// RESTRIO do algoritmo de SHOR #2:
		// + O nmero no pode ser PRIMO
		if (testarPrimalidade(NUMERO)) 
		{
			ShorTask.setMensagem("AVISO #2: o numero precisa ser no primo!\n");
			erro = true;
			return false;
		}
		  
		// RESTRIO do algoritmo de SHOR #3:
		// + O nmero no pode ser uma POTNCIA DE PRIMOS
		if (testarPotenciaPrimos(NUMERO)) 
		{
			ShorTask.setMensagem("AVISO #3: o numero precisa nao ser uma potencia de primos!\n");
			erro = true;
			return false;
		}

		// Precisamos achar, de acordo com o algoritmo de SHOR, um inteiro X
		// que seja CO-PRIMO do numero a ser fatorado. OBS.: 1 no  um nmero
		// til para o algoritmo
		int x = 0;
		x = 1 + (int)((NUMERO-1)*rand.nextDouble());

		while (MDC(NUMERO,x) != 1 || x == 1) 
			x = 1 + (int)((NUMERO-1)*rand.nextDouble());
		  
		ShorTask.setMensagem("X achado e: " + x + ". OBS.: X e co-primo de " + NUMERO + "\n");

		// Agora precisamos estabelecer um registrador quntico para armazenar
		// todos os estados superpostos de 0 a q-1. Onde q  a potncia de dois
		// tal que n^2 <= q < 2n^2
		int q;
		q = GetQ(NUMERO);
		
		ShorTask.setMensagem("Q achado e: " + q + ". \n"
							 + "OBS.: q eh a potencia de 2 tal que n^2 <= q < 2n^2\n\n");
		
		// -1 porque uma posio  para controle
		Registradores reg1 = new Registradores(tamanhoRegistrador(q) - 1);
		ShorTask.setMensagem("Criado registrador 1 com tamanho = " + reg1.tamanho() + "\n");
		// Setando o Registrador 1 para ser exibido na tela
		ShorTask.setEstadosObjeto(reg1, "CRIADO COM TAMANHO # " + reg1.tamanho(), 1);

		// Este vetor ir reter os valores de q produzidos por x^q mod n
		// Isto  necessrio para reter os valores para us-los quando ns
		// colapsarmos o registrador 1 aps sua medio. Em um computador 
		// quntico real este registrador seria embaralhado, e ento este 
		// registrador EXTRA no seria necessrio. 
		int modex[];
		modex = new int[q];     
		
		// Retem as probabilidades de amplitude dos estados do registrador 1,
		// depois que o registrador 2 foi medido e usado  utilizado para 
		// colocar o registrador 1 em um estado consistente novamente
		Complexos colapso[];
		colapso = new Complexos[q];

		for (int j = 0; j < colapso.length; j++)
			colapso[j] = new Complexos();

		// Apenas um numero complexo temporrio
		Complexos tmp = new Complexos();

		// Este  um vetor de probabilidades de amplitude para nosso
		// segundo registrador quntico. Preenchido com os resultados
		// de x^a mod n
		int i = (int)Math.pow(2,tamanhoRegistrador(NUMERO));
		Complexos mdx[];
		mdx = new Complexos[i]; 

		for (int j = 0; j < mdx.length; j++)
			mdx[j] = new Complexos();

		// Este  o segundo registrador propriamente dito
		Registradores reg2 = new Registradores(tamanhoRegistrador(NUMERO)); 
		ShorTask.setMensagem("Criado registrador 2 com tamanho = " + reg2.tamanho() + "\n");
		ShorTask.setEstadosObjeto(reg2, "CRIADO COM TAMANHO # " + reg2.tamanho(), 2);
		int tmpVal;
		int valor;
		
		// C  algum mltiplo lambda de q/r, onde q  o estado quntico
		// e r  o perodo que ns estamos achando para o fator NUMERO. 
		// M  o valor medido no registrador 1 depois da transformao 
		// de FOURIER
		double c,m; 
		//  usado para armazenar a frao p/denominador onde p/denominador  a melhor
		// aproximao para c com denominador <= q
		int denominador;
		// Armazena o numerador de p/denominador onde p/denominador  a melhor aproximao
		// para c com denominador <= q
		int numerador;
		// Os inteiros [e,a,b] so usados no fim do algoritmo quando se tenta 
		// calcular os fatores de n dados os perodos medidos. FATOR  o fator
		// achado pelo processo.
		int e,a,b;
		int FATOR = 0;
		boolean termino = false;
		// O algoritmo de SHOR falha algumas vezes. Padronizamos que aps
		// 6 falhas o processo deve ser abortado
		while (!termino && tentativas <= 6) 
		{
			ShorTask.setMensagem("\nTENTATIVA # " + tentativas + "\n");

			// Preenchendo o registrador 1 com dados pares superpostos
			reg1.setMedia(q - 1);
			// Atualizao do Objeto registrador na GUI
			ShorTask.setEstadosObjeto(reg1, "APS MDIA REAL", 1);
			// Agora faremos a exponenciao modular sobre os elementos
			// superpostos do registrador 1. Isto , estaremos fazendo
			// x^a mod n, mas explorando o paralelismo quntico um computador
			// quntico poderia fazer isto em um passo, ao invs de calcular
			// cada valor possvel e medir o valor no registrador 1.
			// O resultado  armazenado em um novo registrador, reg2, que 
			// emaranhado (entangled) com o primeiro registrador. Isto significa
			// que quando um for medido, e colapsar seus estados base, o outro
			// precisa estar colapsado dentro de uma superposio de estados
			// consistentes com o valor medido no primeiro registrador. 
			// O tamanho da exponencio modular, ou melhor, do resultado desta
			// operao ser no mximo n, assim o nmero de bits que ns
			// precisaremos ser menor ou igual a log2(n). Neste ponto ns
			// precisamos, tambm, manter um vetor com cada estado produzido
			// quando aplicamos a operao modular. Isto  porque estes registradores,
			// atualmente, so emaranhados em um computador quntico REAL, esta
			// informao  necessria, quando, mais tarde, o primeiro registrador
			// for colapsado.
			
			// Utilizado para aumentar nossa probabilidade de amplitude
			// "probability amplitude"
			tmp.set(1,0);

			// Este loop percorre o intervalo sobre q e coloca os valores
			// de x^a mod n em modex[a]. Isto tambm aumenta a probabilidade
			// de amplitude dos valores de mdx[x^a mod n] em nosso vetor
			// de probabilidades complexas
			for (int j = 0; j < q; j++) 
			{
			  tmpVal = modexp(x,j,NUMERO);
			  modex[j] = tmpVal;
			  mdx[tmpVal] = mdx[tmpVal].soma(tmp);
			}

			// Seta o estado do registrador 2
			reg2.setEstado(mdx);
			ShorTask.setEstadosObjeto(reg2, "APS TESTES DE CONSISTNCIA", 2);
			// Normaliza o registrador 2, assim a medio da probabilidade
			// de um estado  dado por somar as reas de sua probabilidade 
			// de amplitude.
			reg2.normalizar();
			ShorTask.setEstadosObjeto(reg2, "APS NORMALIZAO", 2);
			// Medindo o registrador 2.
			valor = reg2.medicao();
			ShorTask.setEstadosObjeto(reg2, "APS MEDIR OS ESTADOS", 2);

			// Usando a informao do vetor "modex" para colapsar o registrador
			// 1 dentro de um estado consistente com os valores medidos no 
			// registrador 2.
			for (int j = 0 ; j < q ; j++) 
			{
				if (modex[j] == valor) 
					colapso[j].set(1,0);
				else 
					colapso[j].set(0,0);
			}

			// DESABILITADO POR QUESTES DE PERFORMANCE
			// ShorTask.setEstadosObjeto(colapso, "PROBABILIDADES DE AMPLITUDE #1", 3);
			// ShorTask.setEstadosObjeto(mdx, "PROBABILIDADES DE AMPLITUDE #2", 4);
			// ShorTask.setEstadosObjeto(modex, "MDULOS CALCULADOS", 5);

			// Setando o estado do registrador 1 para estar consistente
			// com os valores medidos no registrador 2 e, aps, normalizar
			// as probabilidade de amplitude.
			reg1.setEstado(colapso);
			ShorTask.setEstadosObjeto(reg1, "APS TESTES DE CONSISTNCIA", 1);
			reg1.normalizar();
			ShorTask.setEstadosObjeto(reg1, "APS NORMALIZAO", 1);
			// Efetuando a transformao de FOURIER
			transformacaoFourier(reg1, q);
			ShorTask.setEstadosObjeto(reg1, "APS ANLISE DE FOURIER", 1);
			// Medindo o registrador 1, devido a TDF o nmero que ns
			// medimos, m ser algum mltiplo de lambda/r, onde lambda
			//  um inteiro e r  o perodo desejado.
			m = reg1.medicao();
			ShorTask.setEstadosObjeto(reg1, "APS MEDIR OS ESTADOS", 1);
			// Se nada deu errado estamos preparados para encerrar o algoritmo
			termino = true;
			// Se ns medirmos ZERO, precisamos pegar a nova informao
			// sobre o perodo e comear novamente.
			if (m == 0) 
			{
				ShorTask.setMensagem("AVISO #5: medido 0 (Zero). Temos uma falha!\n");
				termino = false;
			}
			// A rotina de medio tem -1 como um cdigo de ERRO, devido
			// aos erros de arredondamento a medio de um estado pode
			// ocasionalmente falhar gerando este cdigo de erro
			if (m == -1) 
			{
				ShorTask.setMensagem("AVISO #6: falha de arredondamento detectada. Tentando de novo.\n");
				termino = false;
			}
			
			// Se tudo est correndo bem precisamos determinar o perodo de 
			// nossa funo e "pegar" os fatores de NUMERO.
			if (termino) 
			{
				// Agora c =~ lambda / r para algum inteiro lambda. 
				// Este trecho de cdigo  emprestado de BERHNARD OHPNER
                c = (double)m  / (double)q;

              System.out.println("m = " + m + " q = " + q + " c = " + c);

				// Calcula o denominador da melhor aproximao racional
				// para c com den < q. Desde que c  lambda / r para algum
				// inteiro lambda, isto ir nos providenciar o PERODO da 
				// funo, embora este seja por HIPOTTICO, por enquanto.
				denominador = denominador(c, q);

				// Dado o denominador calcular o numerador
				numerador = (int)Math.floor(denominador * c + 0.5);
			  
                System.out.println("Den = " + denominador + " num = " + numerador);

                ShorTask.setMensagem("\nMedido " + m + ", aproximacao para " + c + " e "
								 + numerador + " / " + denominador + "\n");

				// O denominador , dessa forma, o perodo buscado. 
				// Entretando um perodo mpar no  til como um resultado
				// para o algoritmo de SHOR. Se o denominador vezes 2 ainda 
				//  menor que q ns podemos utiliz-lo.
				if (denominador % 2 == 1 && 2 * denominador < q )
				{
					ShorTask.setMensagem("Denominador impar, expandindo por 2\n");
					numerador = 2 * numerador; 
					denominador = 2 * denominador;
				}

				// Inicializao de variveis auxiliares
				e = a = b = FATOR = 0;
			  
				// Como dito, falha se o denominador  mpar
				if (denominador % 2 == 1)  
				{          
					ShorTask.setMensagem("AVISO #7: periodo impar encontrado. Tentando de novo.\n");
					termino = false;
				} 
				else 
				{
					ShorTask.setMensagem("Periodo possivel eh: " + denominador + "\n");
					// Calculando possveis fatores comuns para NUMERO
					e = modexp(x, denominador / 2, NUMERO);    
					a = (e + 1) % NUMERO;     
					b = (e + NUMERO - 1) % NUMERO;          
					
					ShorTask.setMensagem("\nFatores de " + NUMERO + " achados:\n" 
                                       + x + "^" + denominador / 2 + " + 1 mod " + NUMERO + " = " + a 
								       + ",\n" + x + "^" + denominador / 2 + " - 1 mod " + NUMERO 
								       + " = " + b + "\n\n"); 

					FATOR = maior(MDC(NUMERO,a),MDC(NUMERO,b));
				}
			}
			// O MDC, mximo divisor comum, retorna -1 se houve tentativa
			// de calcular o MDC de nmeros onde o mdulo entre eles  ZERO
			if (FATOR == -1) 
			{
				ShorTask.setMensagem("AVISO #8: no calculo [n mod 0] p/ algum n. Tentando de novo.\n");
				termino = false;
			}

			if ((FATOR == NUMERO || FATOR == 1) && termino) 
			{
				ShorTask.setMensagem("AVISO #9: fatores triviais achados 1 e " + NUMERO 
									+ ".  Tentando de novo.\n");
				termino = false;
			}
			
			// Se nada deu ERRADO, ns temos os fatores de NUMERO.
			// Caso contrrio comeamos novamente.
			if (FATOR != 0 && termino)
			{
				ShorTask.setMensagem("Finalmente basta fazermos: FATOR e NUMERO / FATOR" 
						+ "\npara termos os fatores de " + NUMERO 
						+ "\nLogo " + NUMERO + " = " + FATOR + " * " + NUMERO / FATOR + "\n");		
			}
			else if (termino) 
			{
				ShorTask.setMensagem("AVISO #10: achado fator 0. Tentando de novo.\n");
				termino = false;
			}
			
			tentativas++;
		}

		if (tentativas >= 6)
		{
			ShorTask.setMensagem("\nAVISO ##: LIMITE MXIMO DE EXECUES ATINGIDO\n");
			ShorTask.setStatus(100);
			return false;
		}
		else
		{
			ShorTask.setStatus(100);
			return true;
		}
	}

	// Permite a execuo do algoritmo numa THREAD
	public void run()
	{
		if (!erro)
		{
			if (executarAlgoritmo())
				System.err.println("SUCESSO na execucao");
			else
			{
				System.err.println("FALHA na execucao");
				ShorTask.setStatus(100);
			}
		}
	}
}
